@@ -614,7 +614,25 @@ void labeltext(int keypressed, char *clientdata)
 	 invalidate_netlist(topobject);
 
       }
-      else if (!hasstuff && (eventmode == TEXT3_MODE)) topobject->parts--;
+      else if (!hasstuff && (eventmode == TEXT3_MODE)) {
+	 if (areastruct.editpart < topobject->parts) {
+	    short *sptr = allocselect();
+	    *sptr = areastruct.editpart;
+
+	    /* Force the "delete" undo record to be a continuation of	*/
+	    /* the undo series containing the edit.  That way, "undo"	*/
+	    /* does not generate a label with null text.		*/
+
+	    xobjs.undostack->idx = -xobjs.undostack->idx;
+	    standard_element_delete(NORMAL);
+	 }
+	 else {
+	    /* Label had just been created; just delete it w/o undo */
+	    freelabel(curlabel->string);
+	    free(curlabel);
+	    topobject->parts--;
+	 }
+      }
 
       if ((!hasstuff) && (eventmode == CATTEXT_MODE)) {  /* can't have null labels! */ 
 	  undo_action();
@@ -624,8 +642,6 @@ void labeltext(int keypressed, char *clientdata)
 	  eventmode = CATALOG_MODE;
       }
       else if (!hasstuff) {
-	  freelabel(curlabel->string);
-	  free(curlabel);
 	  eventmode = NORMAL_MODE;
       }
       else if (eventmode == CATTEXT_MODE) {
@@ -1037,6 +1053,8 @@ void trackspline(xcWidget w, caddr_t clientdata, caddr_t calldata)
    printpos(newpos.x, newpos.y);
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*--------------------------*/
@@ -1174,6 +1192,8 @@ void trackarc(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*----------------------*/
@@ -1265,6 +1285,8 @@ void trackbox(xcWidget w, caddr_t clientdata, caddr_t calldata)
 
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*----------------------------------*/
@@ -1291,6 +1313,8 @@ void trackwire(xcWidget w, caddr_t clientdata, caddr_t calldata)
       areastruct.save.y = newpos.y;
       printpos(newpos.x, newpos.y);
    }
+
+   flusharea();
 }
 
 /*--------------------------*/
@@ -1465,6 +1489,8 @@ void trackpoly(xcWidget w, caddr_t clientdata, caddr_t calldata)
    printpos(newpos.x, newpos.y);
    areastruct.save.x = newpos.x;
    areastruct.save.y = newpos.y;
+
+   flusharea();
 }
 
 /*-------------------------------------------------*/
