@@ -424,11 +424,9 @@ float UTopScale()
 
 /*------------------------------------------------------------------------*/
 
-short UTopTransScale(float length)
+float UTopTransScale(float length)
 {
-   /* 0.5 gives nearest int but unfortunately gives odd results. . .*/
-  
-   return (short)(length * UTopScale() + 0.45);
+   return (float)(length * UTopScale());
 }
 
 /*----------------------------------------------------------------------*/
@@ -925,6 +923,26 @@ void labelbbox(labelptr labox, XPoint *npoints, objinstptr callinst)
 		labox->scale, labox->rotation); 
 }
 
+/*----------------------------------------------------------------------*/
+/* Calculate the bounding box of a graphic image			*/
+/*----------------------------------------------------------------------*/
+
+void graphicbbox(graphicptr gp, XPoint *npoints)
+{
+   XPoint points[4];
+   int hw = gp->source->width >> 1;
+   int hh = gp->source->height >> 1;
+
+   points[1].x = points[2].x = hw;
+   points[0].x = points[3].x = -hw;
+
+   points[0].y = points[1].y = -hh;
+   points[2].y = points[3].y = hh;
+
+   UTransformPoints(points, npoints, 4, gp->position,
+		gp->scale, gp->rotation); 
+}
+
 /*--------------------------------------------------------------*/
 /* Wrapper for single call to calcbboxsingle() in the netlister */
 /*--------------------------------------------------------------*/
@@ -963,21 +981,28 @@ void calcbboxsingle(genericptr *bboxgen, objinstptr thisinst,
 	 break;
 
       case(LABEL):
-	 {
-	    /* because a pin is offset from its position point, include */
-	    /* that point in the bounding box.				*/
+	 /* because a pin is offset from its position point, include */
+	 /* that point in the bounding box.				*/
 
-	    if (TOLABEL(bboxgen)->pin) {
-               bboxcalc(TOLABEL(bboxgen)->position.x, llx, urx);
-               bboxcalc(TOLABEL(bboxgen)->position.y, lly, ury);
-	    }
-	    labelbbox(TOLABEL(bboxgen), npoints, thisinst);
+	 if (TOLABEL(bboxgen)->pin) {
+            bboxcalc(TOLABEL(bboxgen)->position.x, llx, urx);
+            bboxcalc(TOLABEL(bboxgen)->position.y, lly, ury);
+	 }
+	 labelbbox(TOLABEL(bboxgen), npoints, thisinst);
 
-            for (j = 0; j < 4; j++) {
-               bboxcalc(npoints[j].x, llx, urx);
-               bboxcalc(npoints[j].y, lly, ury);
-            }
-         } break;
+         for (j = 0; j < 4; j++) {
+            bboxcalc(npoints[j].x, llx, urx);
+            bboxcalc(npoints[j].y, lly, ury);
+         }
+         break;
+
+      case(GRAPHIC):
+	 graphicbbox(TOGRAPHIC(bboxgen), npoints);
+         for (j = 0; j < 4; j++) {
+            bboxcalc(npoints[j].x, llx, urx);
+            bboxcalc(npoints[j].y, lly, ury);
+         }
+	 break;
 
       case(PATH): {
 	 genericptr *pathc;
@@ -1358,6 +1383,7 @@ void zoomview(xcWidget w, caddr_t clientdata, caddr_t calldata)
 	 eventmode == FONTCAT2_MODE) {
 
       centerview(areastruct.topinstance);
+      invalidate_graphics(topobject);
       areastruct.lastbackground = NULL;
       renderbackground();
       refresh(NULL, NULL, NULL);
@@ -1375,7 +1401,7 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
    UTransformbyCTM(DCTM, pt1, &newpt1, 1);
    UTransformbyCTM(DCTM, pt2, &newpt2, 1);
 
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc,
 		newpt1.x, newpt1.y, newpt2.x, newpt2.y); 
 } 
 
@@ -1383,10 +1409,9 @@ void UDrawSimpleLine(XPoint *pt1, XPoint *pt2)
 
 void UDrawLine(XPoint *pt1, XPoint *pt2)
 {
-   short tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
+   float tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
 
-   XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ? (int)tmpwidth : 
-	0, LineSolid, CapRound, JoinBevel);
+   SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid, CapRound, JoinBevel);
    UDrawSimpleLine(pt1, pt2); 
 } 
 
@@ -1409,7 +1434,7 @@ void UDrawCircle(XPoint *upt, u_char which)
    XPoint wpt;
 
    user_to_window(*upt, &wpt);
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
 
    switch(which) {
       case P_POSITION_X:
@@ -1437,10 +1462,10 @@ void UDrawCircle(XPoint *upt, u_char which)
 
 void UDrawXAt(XPoint *wpt)
 {
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x - 3,
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x - 3,
 		wpt->y - 3, wpt->x + 3, wpt->y + 3);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x + 3,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wpt->x + 3,
 		wpt->y - 3, wpt->x - 3, wpt->y + 3);
 }
 
@@ -1577,8 +1602,8 @@ void UDrawTextLine(labelptr curlabel, short tpos)
    UPreMultCTM(DCTM, curlabel->position, curlabel->scale, curlabel->rotation);
    tmpjust = flipadjust(curlabel->justify);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
 
    tmpext = ULength(curlabel->string, areastruct.topinstance, 0.0, tpos, NULL);
    xdist = tmpext.width;
@@ -1627,23 +1652,23 @@ void UDrawXLine(XPoint opt, XPoint cpt)
 {
    XPoint upt, vpt;
 
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   XSetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetFunction(dpy, areastruct.gc, GXxor);
 
    user_to_window(cpt, &upt);
    user_to_window(opt, &vpt);
 
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineOnOffDash, CapButt, JoinMiter);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, vpt.x, vpt.y, upt.x, upt.y);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineOnOffDash, CapButt, JoinMiter);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, vpt.x, vpt.y, upt.x, upt.y);
 
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x - 3, upt.y - 3,
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapButt, JoinMiter);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x - 3, upt.y - 3,
 		upt.x + 3, upt.y + 3);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x + 3, upt.y - 3,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, upt.x + 3, upt.y - 3,
 		upt.x - 3, upt.y + 3);
 
-   XSetFunction(dpy, areastruct.gc, areastruct.gctype);
-   XSetForeground(dpy, areastruct.gc, areastruct.gccolor);
+   SetFunction(dpy, areastruct.gc, areastruct.gctype);
+   SetForeground(dpy, areastruct.gc, areastruct.gccolor);
 }
 
 /*-------------------------------------------------------------------------*/
@@ -1655,17 +1680,17 @@ void UDrawBox(XPoint origin, XPoint corner)
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   XSetFunction(dpy, areastruct.gc, GXxor);
-   XSetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
-   XSetLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
+   SetFunction(dpy, areastruct.gc, GXxor);
+   SetForeground(dpy, areastruct.gc, AUXCOLOR ^ BACKGROUND);
+   SetThinLineAttributes(dpy, areastruct.gc, 0, LineSolid, CapRound, JoinBevel);
 
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
 		worig.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
 		wcorn.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
 		wcorn.x, worig.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
 
@@ -1688,14 +1713,14 @@ void UDrawBBox()
    user_to_window(origin, &worig);
    user_to_window(corner, &wcorn);
 
-   XSetForeground(dpy, areastruct.gc, BBOXCOLOR);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
+   SetForeground(dpy, areastruct.gc, BBOXCOLOR);
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, worig.y,
 		worig.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, worig.x, wcorn.y,
 		wcorn.x, wcorn.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, wcorn.y,
 		wcorn.x, worig.y);
-   XDrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
+   DrawLine(dpy, areastruct.areawin, areastruct.gc, wcorn.x, worig.y,
 		worig.x, worig.y);
 }
 
@@ -1707,50 +1732,50 @@ void strokepath(XPoint *pathlist, short number, short style, float width)
 {
    char         solidpart;
    char         dashstring[3];
-   short        tmpwidth;
+   float        tmpwidth;
+   short	minwidth;
 
-   tmpwidth = max(1, UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth *
-	width));
+   tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth * width);
+   minwidth = max(1, (short)tmpwidth);
 
    if (style & FILLED || (!(style & FILLED) && style & OPAQUE)) {
       if ((style & FILLSOLID) == FILLSOLID)
-         XSetFillStyle(dpy, areastruct.gc, FillSolid);
+         SetFillStyle(dpy, areastruct.gc, FillSolid);
       else if (!(style & FILLED)) {
-         XSetFillStyle(dpy, areastruct.gc, FillOpaqueStippled); 
-	 XSetStipple(dpy, areastruct.gc, STIPPLE[7]);
+         SetFillStyle(dpy, areastruct.gc, FillOpaqueStippled); 
+	 SetStipple(dpy, areastruct.gc, 7);
       }
       else {
 	 if (style & OPAQUE)
-            XSetFillStyle(dpy, areastruct.gc, FillOpaqueStippled);
+            SetFillStyle(dpy, areastruct.gc, FillOpaqueStippled);
 	 else
-            XSetFillStyle(dpy, areastruct.gc, FillStippled);
-         XSetStipple(dpy, areastruct.gc, STIPPLE[ (style &
-                FILLSOLID) >> 5] );
+            SetFillStyle(dpy, areastruct.gc, FillStippled);
+         SetStipple(dpy, areastruct.gc, ((style & FILLSOLID) >> 5));
       }
-      XFillPolygon(dpy, areastruct.areawin, areastruct.gc, pathlist, number, Nonconvex,
+      FillPolygon(dpy, areastruct.areawin, areastruct.gc, pathlist, number, Nonconvex,
 		CoordModeOrigin);
       /* return to original state */
-      XSetFillStyle(dpy, areastruct.gc, FillSolid);
+      SetFillStyle(dpy, areastruct.gc, FillSolid);
    }
    if (!(style & NOBORDER)) {
       /* set up dots or dashes */
-      if (style & DASHED) solidpart = (char)(4 * tmpwidth);
-      else if (style & DOTTED) solidpart = (char)tmpwidth;
-      sprintf(dashstring, "%c%c", solidpart, (char)(4 * tmpwidth));
+      if (style & DASHED) solidpart = (char)(4 * minwidth);
+      else if (style & DOTTED) solidpart = (char)minwidth;
+      sprintf(dashstring, "%c%c", solidpart, (char)(4 * minwidth));
       if (style & (DASHED | DOTTED)) {
-         XSetDashes(dpy, areastruct.gc, 0, dashstring, 2);
-         XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ?
-            (int)tmpwidth : 0, LineOnOffDash, CapButt, JoinBevel);
+         SetDashes(dpy, areastruct.gc, 0, dashstring, 2);
+         SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineOnOffDash,
+		CapButt, JoinBevel);
       }
       else
-         XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ?
-            (int)tmpwidth : 0, LineSolid, CapRound, JoinBevel);
+         SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid,
+		CapRound, JoinBevel);
 
       /* draw the spline and close off if so specified */
-      XDrawLines(dpy, areastruct.areawin, areastruct.gc, pathlist,
+      DrawLines(dpy, areastruct.areawin, areastruct.gc, pathlist,
 		number, CoordModeOrigin);
       if (!(style & UNCLOSED))
-         XDrawLine(dpy, areastruct.areawin, areastruct.gc, pathlist[0].x,
+         DrawLine(dpy, areastruct.areawin, areastruct.gc, pathlist[0].x,
 		pathlist[0].y, pathlist[number - 1].x, pathlist[number - 1].y);
    }
 }
@@ -1858,7 +1883,7 @@ void UDrawPath(pathptr thepath)
 void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr *stack)
 {
    genericptr	*areagen;
-   short	tmpwidth;
+   float	tmpwidth;
    int		defaultcolor = passcolor;
    int		curcolor = passcolor;
    int		thispart;
@@ -1896,8 +1921,8 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
      /* draw all of the elements */
    
      tmpwidth = UTopTransScale(xobjs.pagelist[areastruct.page]->wirewidth);
-     XSetLineAttributes(dpy, areastruct.gc, tmpwidth >= 2.0 ? (int)tmpwidth :
-		0, LineSolid, CapRound, JoinBevel);
+     SetLineAttributes(dpy, areastruct.gc, tmpwidth, LineSolid, CapRound,
+		JoinBevel);
 
      /* guard against plist being regenerated during a redraw by the	*/
      /* expression parameter mechanism (should that be prohibited?)	*/
@@ -1933,6 +1958,10 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 	  case(PATH):
 	     UDrawPath(TOPATH(areagen));
 	     break;
+
+	  case(GRAPHIC):
+	     UDrawGraphic(TOGRAPHIC(areagen));
+	     break;
    
           case(OBJINST):
 	     if (areastruct.editinplace && stack && (TOOBJINST(areagen)
@@ -1958,6 +1987,7 @@ void UDrawObject(objinstptr theinstance, short level, int passcolor, pushlistptr
 			TOLABEL(areagen)->pin != INFO && areastruct.pinpointon)
 		UDrawXDown(TOLABEL(areagen));
 	     break;
+
        }
      }
 
