@@ -52,6 +52,7 @@ extern int xctcl_rotate();
 extern int xctcl_flip();
 extern int xctcl_set();
 extern int xctcl_page();
+extern int xctcl_spice();
 extern int xctcl_zoom();
 extern int xctcl_pan();
 extern int xctcl_element();
@@ -61,6 +62,7 @@ extern int xctcl_instance();
 extern int xctcl_spline();
 extern int xctcl_path();
 extern int xctcl_arc();
+extern int xctcl_graphic();
 extern int xctcl_object();
 extern int xctcl_here();
 extern int xctcl_start();
@@ -115,11 +117,13 @@ static cmdstruct xc_commands[] =
    {"spline", (void *)xctcl_spline},
    {"path", (void *)xctcl_path},
    {"arc", (void *)xctcl_arc},
+   {"graphic", (void *)xctcl_graphic},
    {"object", (void *)xctcl_object},
    {"here", (void *)xctcl_here},
    {"netlist", (void *)xctcl_netlist},
    {"symbol", (void *)xctcl_symschem},
    {"schematic", (void *)xctcl_symschem},
+   {"spice", (void *)xctcl_spice},
    {"", NULL}  /* sentinel */
 };
 
